﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace WpfApplication1
{
    //Each property (except the Date) has an associated IsPropertyNameDirty property.
    public class EntityViewModel:INotifyPropertyChanged
    {
        private readonly Entity entity;
        public EntityViewModel(Entity entity)
        {
            this.entity = entity;
        }

        public DateTime Date
        {
            get { return entity.Date; }
            set
            {
                if (entity.Date != value)
                {
                    entity.Date = value;
                    OnPropChanged("Date");
                }
            }
        }

        //This is needed strictly to update the MultiBinding.
        //It has nothing to really do with the cell's changing color.
        //However, it does allow you to create a single CellValuePresenter 
        //style instead of one per property
        private bool isEntityDirty;
        public bool IsEntityDirty
        {
            get { return isEntityDirty; }
            set
            {
                isEntityDirty = value;
                OnPropChanged("IsEntityDirty");
            }
        }

        public int Field1
        {
            get { return entity.Field1; }
            set
            {
                if (entity.Field1 != value)
                {
                    entity.Field1 = value;
                    OnPropChanged("Field1");

                    IsPropertyDirtyHelper("Field1");
                }
            }
        }

        public int Field2
        {
            get { return entity.Field2; }
            set
            {
                if (entity.Field2 != value)
                {
                    entity.Field2 = value;
                    OnPropChanged("Field2");

                    IsPropertyDirtyHelper("Field2");
                }
            }
        }

        public int Field3
        {
            get { return entity.Field3; }
            set
            {
                if (entity.Field3 != value)
                {
                    entity.Field3 = value;
                    OnPropChanged("Field3");

                    IsPropertyDirtyHelper("Field3");
                }
            }
        }

        //I moved all the extra properties into a property indexer and private fields.
        //This doesn't necessarily clean up the view-model, but it allows for a single binding in XAML.
        private bool isField1Dirty;
        private bool isField2Dirty;
        private bool isField3Dirty;

        public bool this[string propNameToCheck]
        {
            get 
            {
                bool retVal = false;
                switch (propNameToCheck)
                { 
                    case "Field1":
                        retVal = isField1Dirty;
                        break;
                    case "Field2":
                        retVal = isField2Dirty;
                        break;
                    case "Field3":
                        retVal = isField3Dirty;
                        break;
                }
                return retVal;
            }
            set 
            {
                switch (propNameToCheck)
                {
                    case "Field1":
                        isField1Dirty = value;
                        break;
                    case "Field2":
                        isField2Dirty = value;
                        break;
                    case "Field3":
                        isField3Dirty = value;
                        break;
                }

                //Just to send notifications to the Multibinding
                IsEntityDirty = !IsEntityDirty;

                //The Item[] string is for the indexer.. not sure what would happen if you have more than one indexer in a class though
                OnPropChanged("Item[]");
            }
        }

        private void IsPropertyDirtyHelper(string propertyName)
        {
            this[propertyName] = true;
            this[propertyName] = false;
        }
        
        
       

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }

        #endregion
    }
}
